;*******************************************************************************
;  MSP430x54x Demo - Block Write and Memory Erase @ 0x10000 Executed from RAM
;
;  Description: This program first copies write_block_int function to RAM.
;  Copying to RAM requires a custom made .xcl command linker file. This file is
;  attached as part of the project called custom_lnk_msp430f5438.xcl. This
;  program then erases Flash address 0x10000 - 0x1FFFF which is executed from
;  RAM. Then, it writes to Bank 1 from 0x10000 to 0x1003F. This write shows the
;  advantage of block writing in which 2 consecutive word can be written.
;
;
;  Important Notes:
;
;  1. Copy a new linker file (custom_lnk_msp430f5438.xcl) before modifying it.
;     See the zip file for the modified linker file. 1 line was added and 2 line
;     was changed. Goto Project Options --> Linker --> Config. Overide the
;     linker command file and insert the new .xcl file.
;
;     -QRAMCODE=FLASHCODE                   // Needed to tell compiler that user
;                                              will copy flash code to ram code
;     -Z(DATA)RAMCODE,DATA16_I,DATA16_Z,DATA16_N,DATA16_HEAP+_DATA16_HEAP_SIZE=1C00-5BFF
;                                           // Add RAMCODE into Flash segment
;     -P(CODE)FLASHCODE,CODE=5C00-FF7F,10000-45BFF
;                                           // Add FLASHCODE into Flash segment
;
;  2. Define the allocated memory area that will be copied from FLASH to RAM.
;     In this case, user has to manually define the start address of FLASH and
;     RAM. These memory addresses has to be the same as defined in the linker
;     file origin address of FLASH_MEM and RAM_MEM. The FLASH_MEM_LENGTH can be
;     changed to however much the final compiled code size is.
;
;     WRITE_LENGTH      EQU   128
;     FLASH_MEM_BEGIN   EQU   0x5C00              ; Flash code starting address
;     FLASH_MEM_LENGTH  EQU   0x0200              ; Function segment size to be copied
;     RAM_MEM_BEGIN     EQU   0x1C00              ; RAM code starting address
;     WRITE_ADDR        EQU   0x10000             ; Flash write address
;
;  RESET the device to re-execute code. This is implemented to prevent
;  stressing of Flash unintentionally.
;  ACLK = REFO = 32kHz, MCLK = SMCLK = default DCO 1048576Hz
;
;                MSP430x54x
;            -----------------
;        /|\|              XIN|-
;         | |                 |
;         --|RST          XOUT|-
;           |                 |
;
;   W. Goh
;   Texas Instruments Inc.
;   July 2011
;   Built with IAR Embedded Workbench Version: 5.20.1
;*******************************************************************************

#include "msp430x54x.h"

#define     LongWord_L  R13
#define     LongWord_H  R14

WRITE_LENGTH      EQU   128
FLASH_MEM_BEGIN   EQU   0x5C00              ; Flash code starting address
FLASH_MEM_LENGTH  EQU   0x0200              ; Function segment size to be copied
RAM_MEM_BEGIN     EQU   0x1C00              ; RAM code starting address
WRITE_ADDR        EQU   0x10000             ; Flash write address

;-------------------------------------------------------------------------------
            RSEG    CSTACK                  ; Define stack segment
;-------------------------------------------------------------------------------
            RSEG    CODE                    ; Assemble to Flash memory
;-------------------------------------------------------------------------------
RESET       mov.w   #0x5C00,SP              ; Initialize stackpointer
StopWDT     mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT

            call    #copy_flash_to_RAM      ; Copy flash to RAM function

            mov.w   #0xF000,LongWord_L      ; initialize Lower Word
            mov.w   #0xF001,LongWord_H      ; initialize Higher Word

            call    #RAM_MEM_BEGIN          ; This portion of code is executed
                                            ; in RAM
Mainloop    jmp     Mainloop                ; Loop forever, SET BREAKPOINT HERE
;-------------------------------------------------------------------------------
copy_flash_to_RAM ; Copy flash function to RAM. Uses Register R15 and R14
;-------------------------------------------------------------------------------
            mov.w   #RAM_MEM_BEGIN,R15      ; Initialize pointers
            movx.a  #FLASH_MEM_BEGIN,R14
memcpy_L1   movx.b  @R14+,0(R15)            ; copy flash function to RAM
            inc.w   R15
            cmpx.a  #FLASH_MEM_BEGIN+FLASH_MEM_LENGTH,R14
            jne     memcpy_L1
            ret
;-------------------------------------------------------------------------------
            RSEG    RAMCODE                    ; Assemble to Flash memory
;-------------------------------------------------------------------------------
write_block_int ; This portion of the code is first stored in Flash and copied
                ; to RAM then finally executes from RAM.
                ; It modifies Register R15, R14, R13
;-------------------------------------------------------------------------------
            movx.a  #WRITE_ADDR,R15         ; Initialize write address
            dint                            ; Errata bug FLASH31 Workaround: Disable global
                                            ; interrupt while erasing. Re-Enable
                                            ; GIE if needed
            ; Erase Flash
loop_L6     bit.w   #BUSY,&FCTL3            ; Check if Flash being used
            jnz     loop_L6                 ;
            mov.w   #FWKEY,&FCTL3           ; Clear Lock bit
            mov.w   #FWKEY+ERASE,&FCTL1     ; Set Erase bit
            mov.w   #0,0(R15)               ; Dummy write to erase Flash seg
loop_L5     bit.w   #BUSY,&FCTL3            ; Check for erase completion
            jnz     loop_L5                 ; Exit if erase is done

            ; Write Flash
            mov.w   #FWKEY+BLKWRT+WRT,&FCTL1 ; Enable block write

loop_L4     cmpx.a  #WRITE_ADDR+WRITE_LENGTH,R15 ; loop while not at end of
            jeq     loop_L1                 ; flash write length.
            mov.w   LongWord_L,0(R15)       ; Write a word to Flash
            mov.w   LongWord_H,2(R15)       ; Write another word to Flash
            addx.a  #0x04,R15               ; Increment Flash Pointer
            incd.w  LongWord_L              ; double Increment value
            incd.w  LongWord_H              ; double Increment value
loop_L2     bit.w   #WAIT,&FCTL3            ; Test wait
            jz      loop_L2                 ; Loop until ready for next byte
            jmp     loop_L4

loop_L1     mov.w   #FWKEY,&FCTL1           ; Clear WRT, BLKWRT
loop_L3     bit.w   #BUSY,&FCTL3            ; Check for write completion
            jnz     loop_L3                 ; Exit if write is done
            mov.w   #FWKEY+LOCK,&FCTL3      ; Set LOCK
            ret
;-------------------------------------------------------------------------------
            COMMON  INTVEC                  ; Interrupt Vectors
;-------------------------------------------------------------------------------
            ORG     RESET_VECTOR            ; POR, ext. Reset
            DW      RESET
            END