;*******************************************************************************
;  MSP430x54x Demo - Block Write and Memory Erase @ 0x10000 Executed from RAM 
;
;  Description: This program first copies write_block_int function to RAM. 
;  Copying to RAM requires a custom made .cmd command linker file. This file is
;  attached as part of the project called custom_lnk_msp430f5438.cmd. This
;  program then erases Flash address 0x10000 - 0x1FFFF which is executed from
;  RAM. Then, it writes to Bank 1 from 0x10000 to 0x1003F. This write shows the
;  advantage of block writing in which 2 consecutive word can be written.
;
;
;  Important Notes:
;
;  1. CCE automatically generates a new copy of linker file in the project
;     directory. The zip file has the attached required modified linker file.
;     See below on what was added to the linker file.
;
;       MEMORY
;       {
;         ...
;         RAM_MEM         : origin = 0x1C00, length = 0x0200
;         FLASH_MEM       : origin = 0x5C00, length = 0x0200
;         ...
;       }
;
;       SECTIONS
;       {
;         ...				    
;         .FLASHCODE : load = FLASH_MEM, run = RAM_MEM
;                                          /* CODE IN FLASH AND WILL BE COPIED
;                                             TO RAM AT EXECUTION HANDLED BY
;                                             USER                            */
;         .RAMCODE   : load = FLASH_MEM    /* CODE WILL BE IN RAM             */
;         ...
;       }
;
;  2. Define the allocated memory area that will be copied from FLASH to RAM. 
;     In this case, user has to manually define the start address of FLASH and
;     RAM. These memory addresses has to be the same as defined in the linker
;     file origin address of FLASH_MEM and RAM_MEM. The FLASH_MEM_LENGTH can be
;     changed to however much the final compiled code size is.
;
;       FLASH_MEM_BEGIN   .equ   0x5C00     ; Flash code starting address
;       FLASH_MEM_LENGTH  .equ   0x0200     ; Function segment size to be copied
;       RAM_MEM_BEGIN     .equ   0x1C00     ; RAM code starting address
;
;  ACLK = REFO = 32kHz, MCLK = SMCLK = default DCO 1048576Hz
;  /* Set Breakpoint in the Mainloop to avoid Stressing Flash */
;
;                MSP430x54x
;            -----------------
;        /|\|              XIN|-
;         | |                 |
;         --|RST          XOUT|-
;           |                 |
;
;   W. Goh
;   Texas Instruments Inc.
;   July 2011
;   Built with Code Composer Studio Version: 4.2.3.00004
;*******************************************************************************
            .cdecls C,LIST, "msp430x54x.h"
;-------------------------------------------------------------------------------
            .def    RESET                   ; Export program entry-point to
                                            ; make it known to linker.
            .global _main

LongWord_L  .set    R13
LongWord_H  .set    R14

WRITE_LENGTH      .equ   128
FLASH_MEM_BEGIN   .equ   0x5C00             ; Flash code starting address
FLASH_MEM_LENGTH  .equ   0x0200             ; Function segment size to be copied
RAM_MEM_BEGIN     .equ   0x1C00             ; RAM code starting address
WRITE_ADDR        .equ   0x10000            ; Flash write address

;-------------------------------------------------------------------------------
            .text                           ; Flash Memory
;-------------------------------------------------------------------------------
_main
RESET       mov.w   #0x5C00,SP              ; Initialize stackpointer
StopWDT     mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT

            call    #copy_flash_to_RAM      ; Copy flash to RAM function

            mov.w   #0xF000,LongWord_L      ; initialize Lower Word
            mov.w   #0xF001,LongWord_H      ; initialize Higher Word
            
Mainloop    call    #RAM_MEM_BEGIN          ; This portion of code is executed
                                            ; in RAM
            jmp     Mainloop                ; Loop forever, SET BREAKPOINT HERE
;-------------------------------------------------------------------------------
copy_flash_to_RAM ; Copy flash function to RAM. Uses Register R15 and R14
;-------------------------------------------------------------------------------
            mov.w   #RAM_MEM_BEGIN,R15      ; Initialize pointers 
            movx.a  #FLASH_MEM_BEGIN,R14
memcpy_L1   movx.b  @R14+,0(R15)            ; copy flash function to RAM
            inc.w   R15
            cmpx.a  #FLASH_MEM_BEGIN+FLASH_MEM_LENGTH,R14
            jne     memcpy_L1
            ret  
;-------------------------------------------------------------------------------
            .sect    ".FLASHCODE"
write_block_int ; This portion of the code is first stored in Flash and copied 
                ; to RAM then finally executes from RAM.
                ; It modifies Register R15, R14, R13
;-------------------------------------------------------------------------------
            movx.a  #WRITE_ADDR,R15         ; Initialize write address
            dint                            ; Errata bug FLASH31 Workaround: Disable global 
                                            ; interrupt while erasing. Re-Enable
                                            ; GIE if needed
            ; Erase Flash
loop_L6     bit.w   #BUSY,&FCTL3            ; Check if Flash being used
            jnz     loop_L6                 ; 
            mov.w   #FWKEY,&FCTL3           ; Clear Lock bit
            mov.w   #FWKEY+ERASE,&FCTL1     ; Set Erase bit
            mov.w   #0,0(R15)               ; Dummy write to erase Flash seg
loop_L5     bit.w   #BUSY,&FCTL3            ; Check for erase completion
            jnz     loop_L5                 ; Exit if erase is done
            
            ; Write Flash
            mov.w   #FWKEY+BLKWRT+WRT,&FCTL1 ; Enable block write  
  
loop_L4     cmpx.a  #WRITE_ADDR+WRITE_LENGTH,R15 ; loop while not at end of
            jeq     loop_L1                 ; flash write length.
            mov.w   LongWord_L,0(R15)       ; Write a word to Flash
            mov.w   LongWord_H,2(R15)       ; Write another word to Flash
            addx.a  #0x04,R15               ; Increment Flash Pointer
            incd.w  LongWord_L              ; double Increment value
            incd.w  LongWord_H              ; double Increment value
loop_L2     bit.w   #WAIT,&FCTL3            ; Test wait
            jz      loop_L2                 ; Loop until ready for next byte
            jmp     loop_L4

loop_L1     mov.w   #FWKEY,&FCTL1           ; Clear WRT, BLKWRT
loop_L3     bit.w   #BUSY,&FCTL3            ; Check for write completion
            jnz     loop_L3                 ; Exit if write is done
            mov.w   #FWKEY+LOCK,&FCTL3      ; Set LOCK 
            ret         
;-------------------------------------------------------------------------------
;			Interrupt Vectors
;-------------------------------------------------------------------------------
            .sect    ".reset"            	; RESET Vector
            .short   RESET                  ;
            .end
            